/*
 * Copyright (C) 2006-2007 by egnite Software GmbH
 * Copyright (C) 2008-2009 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file config.c
 * \brief Radio configuration.
 *
 * \verbatim
 * $Id: config.c 63 2010-08-01 15:31:14Z haraldkipp $
 * \endverbatim
 */

#include "config.h" /* Must be first. */

#include <cfg/eeprom.h>
#if defined(NUT_CONFIG_AT45D_CS)
#define DATAFLASH_CS    NUT_CONFIG_AT45D_CS
#else
#define DATAFLASH_CS    0
#endif

#include <dev/spi_at45d.h>

#include <sys/confos.h>
#include <sys/confnet.h>
#include <sys/nutdebug.h>

#include <fs/rawfs.h>

#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>

#include <arpa/inet.h>

#include "logmsg.h"
#include "favlist.h"
#include "proxy.h"
#include "webradio.h"


/*!
 * \brief HTTP proxy configuration.
 */
HTTP_PROXY proxy;

static int cfg_volid;

/*!
 * \brief Reset configuration to factory defaults.
 */
void ConfigResetFactory(void)
{
    LogMsg(LOG_WARN | LOG_CONFIG, "Factory reset\n");
    /* Initial radio control settings. */
    webradio.wr_gain = -12;
    webradio.wr_pridns = confnet.cdn_gateway;
    webradio.wr_secdns = 0;
    memset(&proxy, 0, sizeof(proxy));
}

#if USE_CONFIG_RAWFS

/*!
 * \brief Calculate total size of configuration data.
 *
 * Should be used to avoid configuration data overflow. Untested.
 *
 * \return Number of bytes used by configuration data.
 */
size_t ConfigSize(void)
{
    size_t rc = 0;
    int idx;
    int i;
    RADIOSTATION *flp;

    for (idx = 1; idx < MAX_FAVORITES; idx++) {
        flp = &favlist[idx];
        if (flp->rs_name) {
            rc += strlen(flp->rs_name) + 1;
            for (i = 0; i < flp->rs_streams; i++) {
                if (flp->rs_uri[i]) {
                    rc += strlen(flp->rs_uri[i]) + 1;
                }
            }
        }
    }
    return rc + 1;
}

size_t ConfigLoadString(FILE *fp, char * str, size_t size)
{
    size_t rc = 0;
    int ch;

    while (rc < size) {
        ch = fgetc(fp);
        if (ch == 0 || ch == EOF) {
            break;
        }
        str[rc++] = ch;
    }
    str[rc++] = 0;

    return rc;
}

/*!
 * \brief Save configuration in non volatile memory.
 */
int ConfigSave(FILE *fp)
{
    int rc;

    NUTASSERT(fp != NULL);

    LogDbgMsg(LOG_CONFIG, "Save config --------------\n");
    /* Save our name. */
    fseek(fp, NVMEM_CONFIG_POS, SEEK_SET);
    fwrite(CONFIG_MAGIC, 1, sizeof(CONFIG_MAGIC), fp);

    /* Save radio control. */
    fwrite(&webradio.wr_gain, sizeof(webradio.wr_gain), 1, fp);
    fwrite(&webradio.wr_pridns, sizeof(webradio.wr_pridns), 1, fp);
    fwrite(&webradio.wr_secdns, sizeof(webradio.wr_secdns), 1, fp);
    fwrite(proxy.proxy_host, 1, strlen(proxy.proxy_host) + 1, fp);
    fwrite(&proxy.proxy_port, sizeof(proxy.proxy_port), 1, fp);
    rc = fflush(fp);
    /* Must use 2 calls because of static buffer in inet_ntoa. */
    LogMsg(LOG_CONFIG, "Saved %d %s", 
        webradio.wr_gain, 
        inet_ntoa(webradio.wr_pridns));
    LogMsg(LOG_CONFIG, " %s %s:%u\n", 
        inet_ntoa(webradio.wr_secdns),
        proxy.proxy_host, proxy.proxy_port);

    return rc;
}

int ConfigLoad(FILE *fp)
{
    int rc = -1;
    char buf[8];

    NUTASSERT(fp != NULL);

    LogDbgMsg(LOG_CONFIG, "Load config --------------\n");
    fseek(fp, NVMEM_CONFIG_POS, SEEK_SET);
    ConfigLoadString(fp, buf, sizeof(buf) - 1);
    if (strcmp(buf, CONFIG_MAGIC) == 0) {
        fread(&webradio.wr_gain, sizeof(webradio.wr_gain), 1, fp);
        fread(&webradio.wr_pridns, sizeof(webradio.wr_pridns), 1, fp);
        fread(&webradio.wr_secdns, sizeof(webradio.wr_secdns), 1, fp);
        ConfigLoadString(fp, proxy.proxy_host, sizeof(proxy.proxy_host));
        fread(&proxy.proxy_port, sizeof(proxy.proxy_port), 1, fp);
        LogMsg(LOG_CONFIG, "Loaded %d, %s", 
            webradio.wr_gain, 
            inet_ntoa(webradio.wr_pridns));
        LogMsg(LOG_CONFIG, ", %s, %s:%u\n", 
            inet_ntoa(webradio.wr_secdns),
            proxy.proxy_host, proxy.proxy_port);
        rc = 0;
    } else {
        LogMsg(LOG_WARN | LOG_CONFIG, "No configuration\n");
    }
    return rc;
}

FILE *ConfigOpen(void)
{
    if (NutRegisterSpiDevice(&devSpiAt45d0, &DEV_SPIBUS, DATAFLASH_CS)) {
        LogMsg(LOG_ERROR | LOG_CONFIG, "DataFlash init failed\n");
        return NULL;
    }
    if (NutRegisterDevice(&devRawFs0, 0, 0)) {
        LogMsg(LOG_ERROR | LOG_CONFIG, "Raw file system init failed\n");
        return NULL;
    }
    if ((cfg_volid = _open("AT45D0:0/RAWFS0", _O_RDWR | _O_BINARY)) == -1) {
        LogMsg(LOG_ERROR | LOG_CONFIG, "Error mounting raw file system\n");
        return NULL;
    }
    LogMsg(LOG_CONFIG, "%ld bytes available for config\n", _filelength(cfg_volid));

    return fopen("RAWFS0:", "r");
}

#else
int ConfigSave(FILE *fp)
{
    return -1;
}
#endif
